package org.apache.maven.continuum.core.action;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import org.apache.continuum.dao.ProjectGroupDao;
import org.apache.maven.continuum.ContinuumException;
import org.apache.maven.continuum.model.project.Project;
import org.apache.maven.continuum.model.project.ProjectGroup;
import org.apache.maven.continuum.store.ContinuumStoreException;

import java.util.Map;

/**
 * @author <a href="mailto:trygvis@inamo.no">Trygve Laugst&oslash;l</a>
 * @version $Id: StoreProjectAction.java 1372260 2012-08-13 04:29:09Z brett $
 * @plexus.component role="org.codehaus.plexus.action.Action"
 * role-hint="store-project"
 */
public class StoreProjectAction
    extends AbstractContinuumAction
{
    private static final String KEY_SCM_USE_CREDENTIALS_CACHE = "useCredentialsCache";

    /**
     * @plexus.requirement
     */
    private ProjectGroupDao projectGroupDao;

    public void execute( Map context )
        throws ContinuumException, ContinuumStoreException
    {
        Project project = getUnvalidatedProject( context );

        ProjectGroup projectGroup = getUnvalidatedProjectGroup( context );

        // ----------------------------------------------------------------------
        //
        // ----------------------------------------------------------------------

        boolean useCredentialsCache = isUseScmCredentialsCache( context, false );
        // CONTINUUM-1605 don't store username/password
        if ( useCredentialsCache )
        {
            project.setScmUsername( null );
            project.setScmPassword( null );
            project.setScmUseCache( true );
        }

        projectGroup.addProject( project );

        projectGroupDao.updateProjectGroup( projectGroup );

        setProjectId( context, project.getId() );

        // ----------------------------------------------------------------------
        // Set the working directory
        // ----------------------------------------------------------------------
/*
        File projectWorkingDirectory = new File( getWorkingDirectory( context ), project.getId() );

        if ( !projectWorkingDirectory.exists() && !projectWorkingDirectory.mkdirs() )
        {
            throw new ContinuumException( "Could not make the working directory for the project " +
                                          "'" + projectWorkingDirectory.getAbsolutePath() + "'." );
        }

        // The working directory is created based on the project id so we can always
        // figure out what it is.

        project.setWorkingDirectory( projectWorkingDirectory.getAbsolutePath() );
*/
//        store.updateProject( project );
    }

    public static boolean isUseScmCredentialsCache( Map<String, Object> context, boolean defaultValue )
    {
        return getBoolean( context, KEY_SCM_USE_CREDENTIALS_CACHE, defaultValue );
    }

    public static void setUseScmCredentialsCache( Map<String, Object> context, boolean useScmCredentialsCache )
    {
        context.put( KEY_SCM_USE_CREDENTIALS_CACHE, useScmCredentialsCache );
    }
}
